import React, { useEffect, useRef } from 'react'
import { transform } from '@babel/standalone'

const LivePreview = ({ code }) => {
  const iframeRef = useRef(null)

  useEffect(() => {
    if (!iframeRef.current) return

    // Aquí, asegúrate de que 'code' contenga solo el JSX, no una declaración de componente.
    // Por ejemplo, debería verse así: "<div>Hello from JSX!</div>;"
    const transformedCode = transform(`<>${code}</>`, {
      presets: ['react']
    }).code

    const iframe = iframeRef.current
    const iframeDoc = iframe.contentDocument || iframe.contentWindow.document
    iframeDoc.open()
    iframeDoc.write(`
      <!DOCTYPE html>
      <html>
      <head><title>Preview</title></head>
      <body>
        <div id="root"></div>
        <script src="https://unpkg.com/react/umd/react.development.js"></script>
        <script src="https://unpkg.com/react-dom/umd/react-dom.development.js"></script>
        <script type="text/javascript">
          ${transformedCode}
          const root = ReactDOM.createRoot(document.getElementById('root'));
          root.render(React.createElement(React.Fragment, null, ...React.Children.toArray(React.createElement(() => (${code}), {}))));
        </script>
      </body>
      </html>
    `)
    iframeDoc.close()
  }, [code])

  return <iframe ref={iframeRef} style={{ width: '100%', height: '300px', border: 'none' }} />
}

export default LivePreview
