import React from 'react'
import { I18n, SearchHeaderProps } from '../KanbanBoard/kanbanBoard.type'
import { Icons } from '../icons'
import { cva } from 'class-variance-authority'

const SearchHeader: React.FC<{ searchHeader: SearchHeaderProps, i18n?: I18n }> = ({ searchHeader, i18n }) => {
  const isMobile = true
  const [isFocusedSearch, setIsFocusedSearch] = React.useState(false)
  const {
    upperButtons,
    hideInput,
    searchHeaderStyles,
    searchHeaderClassName,
    inputSearchClassName,
    inputSearchStyles,
    inputPlaceholder,
    upperRightButtons,
    showNewButton,
    newButtonOnClick,
    newButtonStyles,
    newButtonClassName,
    newButtonText,
    newButtonIcon
  } = searchHeader

  const gridClass = React.useMemo(() => {
    if (upperButtons && upperRightButtons) {
      return 'grid-cols-[minmax(auto,min-content),auto,minmax(auto,min-content),50px] sm:grid-cols-[minmax(auto,min-content),auto,minmax(auto,min-content),100px]'
    } else if (upperButtons || upperRightButtons) {
      return 'grid-cols-[minmax(auto,min-content),auto,50px] sm:grid-cols-[minmax(auto,min-content),auto,100px]'
    } else {
      return 'grid-cols-[auto,50px] sm:grid-cols-[auto,100px]'
    }
  }, [upperButtons, upperRightButtons])

  const variantsButton = cva(
    'flex cursor-pointer items-center justify-center bg-theme-app-500 px-2 py-1 text-white drop-shadow-sm transition-all duration-500 ease-in-out hover:bg-theme-app-600', {
      variants: {
        screenType: {
          desktop: 'rounded-lg',
          mobile: 'rounded-r-lg'
        }
      }
    })

  return (
    <div className={`form-group relative grid w-full gap-1 pb-2 ${searchHeaderClassName} ${gridClass}`} style={searchHeaderStyles}>
      {upperButtons}
      {!hideInput && (
        <div className={`flex w-full ${inputSearchClassName} ${!upperButtons && !upperRightButtons && !showNewButton ? 'col-span-full' : ''}`}>
          <input
            type="search"
            style={inputSearchStyles}
            className='focus:border-r-none flex w-full !rounded-l-md border-y border-l !border-slate-300 px-2 py-1 drop-shadow-sm focus:!border-theme-app-500 focus:!ring-0 dark:bg-[#272727] dark:text-white dark:focus:!border-theme-app-500 dark:focus:!ring-0'
            placeholder={inputPlaceholder}
          />
          <button
            className={variantsButton({
              screenType: isMobile && !isFocusedSearch ? 'mobile' : 'desktop'
            })}
            onClick={() => isMobile && setIsFocusedSearch(!isFocusedSearch)}
          >
            <Icons.search className="m-auto h-4 w-4 text-white" />
          </button>
        </div>
      )}
      {upperRightButtons}
      {showNewButton === true && (
        <button
          type='button'
          className={`ml-1 inline-flex w-auto items-center justify-center rounded-lg bg-emerald-600 px-2 py-0.5 text-center text-sm font-medium text-white drop-shadow-sm hover:bg-emerald-700 focus:outline-none focus:ring-4 focus:ring-emerald-300 md:px-5 dark:bg-emerald-600 dark:hover:bg-emerald-700 dark:focus:ring-emerald-800 ${newButtonClassName}`}
          style={newButtonStyles}
          onClick={() => newButtonOnClick()}
        >
          {newButtonIcon || <Icons.new className='h-5 w-5' />}
          <span className='ml-2 hidden whitespace-nowrap sm:!inline'>
            {newButtonText || (i18n.t('common.newItem') !== 'txtNone' ? i18n.t('common.newItem') : 'New Task')}
          </span>
        </button>
      ) }
    </div>
  )
}

export default SearchHeader
