import { useEffect } from 'react'

const useKeyPress = (targetKeys, onKeyPress, preventDefault = false) => {
  useEffect(() => {
    const handleKeyPress = (event) => {
      const keys = Array.isArray(targetKeys) ? targetKeys : [targetKeys]

      if (keys.every(key => {
        if (key === 'Meta' || key === 'Control' || key === 'Alt' || key === 'Shift') {
          return event.getModifierState(key)
        } else {
          return event.key === key
        }
      })) {
        if (preventDefault) {
          event.preventDefault()
        }
        onKeyPress()
      }
    }

    window.addEventListener('keydown', handleKeyPress)
    return () => {
      window.removeEventListener('keydown', handleKeyPress)
    }
  }, [targetKeys, onKeyPress, preventDefault])
}

export default useKeyPress
